/*
 *	file:		code03e8.c
 *	project:	SysSndWavHack
 *	content:	main functions
 *	updated:	01.07.2001
 *
 *	copyright: Collin R. Mulliner <palm@mulliner.org>
 *	web: www.mulliner.org/palm/
 * 
 */

/*
 *  This file is part of SysSndWavHack
 *
 *  SysSndWavHack is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SysSndWavHack is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SysSndWavHack; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include "sswh.h"
#include "callback.h"

void MySndPlaySystemSound(SndSysBeepType beepID);
Err readWavData(void *dataP, UInt32 offset, UInt32 *sizeP, void *userDataP);
Boolean play_wav(UInt8 beepID);

/*
 *	replacement function call for SndPlaySystemSound(..)
 */
void MySndPlaySystemSound(SndSysBeepType beepID)
{
	void (*old_sndplaysystemsound)(SndSysBeepType);
	UInt32 old_trap, sema;
	
	
	CALLBACK_PROLOGUE
			
	if (FtrGet(CRID, 0, &sema) != 0) {
		// fake semaphore !
		FtrSet(CRID, 0, 0);		
					
		if (beepID != sndClick) {	// only system sounds other "click" are safe to change !
			if (!play_wav(beepID)) {	// can't play wav - call real system call
				// get pointer to real system call
				FtrGet(CRID, trapIDSndPlaySystemSound, &old_trap);
				old_sndplaysystemsound = (void (*)(SndSysBeepType)) old_trap;
				// call the real system call
				old_sndplaysystemsound(beepID);
			}		
		}
		else {	// do "click"
			// get pointer to real system call
			FtrGet(CRID, trapIDSndPlaySystemSound, &old_trap);
			old_sndplaysystemsound = (void (*)(SndSysBeepType)) old_trap;
			// call the real system call
			old_sndplaysystemsound(beepID);
		}
	
		// clear fake semaphore !
		FtrUnregister(CRID, 0);
	}
	
	CALLBACK_EPILOGUE
}

/*
 *	play wav file
 */
Boolean play_wav(UInt8 beepID)
{
	AudioFormatType auData;
	UInt16 percent;
	FileRef fp;
	Int8 index;
	UInt32 version, volIterator;
	Err error;
	UInt16 volRef;
	char filename[30];
	// pathnames for the wav files
	const char pathnames[2][8] = {"/AUDIO/", "/"};
	const char filenames[6][13] = {"info.wav", "warning.wav", "error.wav", "startup.wav", "alarm.wav", "confirm.wav"};

	
	// check for TRG / HandEra Audio Library !
	if (!(_TRGAudioFeaturePresent(&version))) {
		return(false);
	}

	// init volume iterator
	volIterator = vfsIteratorStart;
	// scan all volumes
	while ((error = VFSVolumeEnumerate(&volRef, &volIterator)) == errNone) {
		
	/*	{	// DEBUG !
			char test[20];
			StrIToA(test, volRef);
			WinDrawChars(test, StrLen(test),5,5);
			SysTaskDelay(500);
		} */
		
		// set index to -1
		index = -1;
		do {
			// first time this will become 0 (zero)
			index++;
			// make filename
			StrCopy(filename, pathnames[index]);
			StrCat(filename, filenames[beepID-1]);
		} while ((error = VFSFileOpen(volRef, filename, vfsModeRead, &fp)) != errNone && index < 1);
		if (error == errNone) break;	// jup out of the loop when the file is open!
	}
	
	if (error != errNone) {
		return(false);
	}
	else {
		AudioOpenWave(&auData, readWavData, (void*) &fp);
		AudioPlayData();
		while (AudioTellPercent(&percent) && percent < 100) {
			SysTaskDelay(10);
		}
		AudioCloseWave();

		VFSFileClose(fp);
	}
	
	return(true);
}

/*
 *	read wav data - callback function
 */
Err readWavData(void *dataP, UInt32 offset, UInt32 *sizeP, void *userDataP)
{
	Err ret_val = 0;
	
	
	CALLBACK_PROLOGUE
	
	if ((ret_val = VFSFileSeek(*((FileRef*) userDataP), vfsOriginBeginning, offset)) != errNone) {
		CALLBACK_EPILOGUE
		return(ret_val);
	}
	
	ret_val = VFSFileRead(*((FileRef*) userDataP), *sizeP, dataP, sizeP);
			
	CALLBACK_EPILOGUE
	
	return(ret_val);
}
